@@ -20,10 +20,9 @@ import os.path
 
 from gi.repository import GLib, Gtk, Gdk, Gio, Pango
 
-from gtweak.tweakmodel import Tweak
+from gtweak.tweakmodel import Tweak, TweakGroup
 from gtweak.gsettings import GSettingsSetting, GSettingsFakeSetting, GSettingsMissingError
 from gtweak.gtksettings import GtkSettingsManager
-from gtweak.gconf import GConfSetting
 from gtweak.gshellwrapper import GnomeShellFactory
 
 UI_BOX_SPACING = 4
@@ -43,6 +42,14 @@ def build_label_beside_widget(txt, *widget, **kwargs):
         image.set_tooltip_text(tip)
         return image
 
+    def show_tooltip_when_ellipsized(label, x, y, keyboard_mode, tooltip):
+        layout = label.get_layout()
+        if layout.is_ellipsized():
+            tooltip.set_text(label.get_text())
+            return True
+        else:
+            return False
+
     if kwargs.get("hbox"):
         hbox = kwargs.get("hbox")
     else:
@@ -52,6 +59,8 @@ def build_label_beside_widget(txt, *widget, **kwargs):
     lbl = Gtk.Label(txt)
     lbl.props.ellipsize = Pango.EllipsizeMode.END
     lbl.props.xalign = 0.0
+    lbl.set_has_tooltip(True)
+    lbl.connect("query-tooltip", show_tooltip_when_ellipsized)
     hbox.pack_start(lbl, True, True, 0)
 
     if kwargs.get("info"):
@@ -70,7 +79,7 @@ def build_label_beside_widget(txt, *widget, **kwargs):
     #label. By convention this is true in the great majority of cases. Settings that
     #construct their own widgets will need to set this themselves
     lbl.set_mnemonic_widget(widget[-1])
-
+    
     return hbox
 
 def build_combo_box_text(selected, *values):
@@ -142,6 +151,8 @@ class _GSettingsTweak(Tweak):
         schema_name = adjust_schema_for_overrides(schema_name, key_name, options)
         self.schema_name = schema_name
         self.key_name = key_name
+        if 'uid' not in options:
+            options['uid'] = key_name
         try:
             self.settings = GSettingsSetting(schema_name, **options)
             Tweak.__init__(self,
@@ -159,28 +170,121 @@ class _GSettingsTweak(Tweak):
             self.loaded = False
             logging.info("GSettings missing key %s (key %s)" % (schema_name, key_name))
 
+        if options.get("logout_required") and self.loaded:
+            self.settings.connect("changed::%s" % key_name, self._on_changed_notify_logout)
 
-class GSettingsSwitchTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, **options):
+    def _on_changed_notify_logout(self, settings, key_name):
+        self.notify_logout()
+
+class _DependableMixin(object):
+
+    def add_dependency_on_tweak(self, depends, depends_how):
+        if isinstance(depends, Tweak):
+            self._depends = depends
+            if depends_how is None:
+                depends_how = lambda x,kn: x.get_boolean(kn)
+            self._depends_how = depends_how
+
+            sensitive = self._depends_how(
+                                depends.settings,
+                                depends.key_name,
+            )
+            self.set_sensitive(sensitive)
+
+            depends.settings.connect("changed::%s" % depends.key_name, self._on_changed_depend)
+
+    def _on_changed_depend(self, settings, key_name):
+        sensitive = self._depends_how(settings,key_name)
+        self.set_sensitive(sensitive)
+
+class ListBoxTweakGroup(Gtk.ListBox, TweakGroup):
+    def __init__(self, name, *tweaks, **options):
+        if 'uid' not in options:
+            options['uid'] = self.__class__.__name__
+        Gtk.ListBox.__init__(self,
+                        selection_mode=Gtk.SelectionMode.NONE,
+                        name=options['uid'])
+        self.get_style_context().add_class(
+                        options.get('css_class','tweak-group'))
+        self.props.vexpand = False
+        self.props.valign = Gtk.Align.START
+
+        self._sg = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
+        self._sg.props.ignore_hidden = True
+
+        TweakGroup.__init__(self, name, **options)
+
+        for t in tweaks:
+            self.add_tweak_row(t)
+
+    #FIXME: need to add remove_tweak_row and remove_tweak (which clears
+    #the search cache etc)
+
+    def add_tweak_row(self, t, position=None):
+        if self.add_tweak(t):
+            if isinstance(t, Gtk.ListBoxRow):
+                row = t
+            else:
+                row = Gtk.ListBoxRow(name=t.uid)
+                row.get_style_context().add_class("tweak")
+                if isinstance(t, Title):
+                    row.get_style_context().add_class("title")
+                row.add(t)
+            if position is None:
+                self.add(row)
+            else:
+                self.insert(row, position)
+            if t.widget_for_size_group:
+                self._sg.add_widget(t.widget_for_size_group)
+            return row
+
+class GSettingsCheckTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
+        _GSettingsTweak.__init__(self, schema_name, key_name, **options)
+
+        widget = Gtk.CheckButton.new_with_label(name)
+        self.settings.bind(
+                key_name,
+                widget,
+                "active", Gio.SettingsBindFlags.DEFAULT)
+        self.add(widget)
+        self.widget_for_size_group = None
+
+        self.add_dependency_on_tweak(
+                options.get("depends_on"),
+                options.get("depends_how")
+        )
+
+class GSettingsSwitchTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         w = Gtk.Switch()
         self.settings.bind(key_name, w, "active", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.name, w)
-        # never change the size of a switch
+
+        build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = None
 
-class GSettingsFontButtonTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, **options):
+        self.add_dependency_on_tweak(
+                options.get("depends_on"),
+                options.get("depends_how")
+        )
+
+class GSettingsFontButtonTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         w = Gtk.FontButton()
         self.settings.bind(key_name, w, "font-name", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.name, w)
+        build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = w
 
-class GSettingsRangeTweak(_GSettingsTweak):
+class GSettingsRangeTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
     def __init__(self, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         #returned variant is range:(min, max)
@@ -188,11 +292,13 @@ class GSettingsRangeTweak(_GSettingsTweak):
 
         w = Gtk.HScale.new_with_range(_min, _max, options.get('adjustment_step', 1))
         self.settings.bind(key_name, w.get_adjustment(), "value", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.name, w)
+
+        build_label_beside_widget(self.name, w, hbox=self)
         self.widget_for_size_group = w
 
-class GSettingsSpinButtonTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, **options):
+class GSettingsSpinButtonTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         #returned variant is range:(min, max)
@@ -203,11 +309,13 @@ class GSettingsSpinButtonTweak(_GSettingsTweak):
         w.set_adjustment(adjustment)
         w.set_digits(options.get('digits', 0))
         self.settings.bind(key_name, adjustment, "value", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.name, w)
+
+        build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = w
 
-class GSettingsComboEnumTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, **options):
+class GSettingsComboEnumTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         _type, values = self.settings.get_range(key_name)
@@ -218,10 +326,9 @@ class GSettingsComboEnumTweak(_GSettingsTweak):
         w.connect('changed', self._on_combo_changed)
         self.combo = w
 
-        self.widget = build_label_beside_widget(self.name, w)
+        build_label_beside_widget(name, w, hbox=self)
         self.widget_for_size_group = w
 
-
     def _values_are_different(self):
         #to stop bouncing back and forth between changed signals. I suspect there must be a nicer
         #Gio.settings_bind way to fix this
@@ -242,8 +349,9 @@ class GSettingsComboEnumTweak(_GSettingsTweak):
         if self._values_are_different():
             self.settings.set_string(self.key_name, val)
 
-class GSettingsComboTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, key_options, **options):
+class GSettingsComboTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, key_options, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         #check key_options is iterable
@@ -256,10 +364,11 @@ class GSettingsComboTweak(_GSettingsTweak):
                     self.settings.get_string(self.key_name),
                     *key_options)
         self.combo.connect('changed', self._on_combo_changed)
-        self.widget = build_label_beside_widget(self.name, self.combo)
+        self.settings.connect('changed::'+self.key_name, self._on_setting_changed)
+
+        build_label_beside_widget(name, self.combo,hbox=self)
         self.widget_for_size_group = self.combo
 
-        self.settings.connect('changed::'+self.key_name, self._on_setting_changed)
 
     def _on_setting_changed(self, setting, key):
         assert key == self.key_name
@@ -278,50 +387,6 @@ class GSettingsComboTweak(_GSettingsTweak):
             value = combo.get_model().get_value(_iter, 0)
             self.settings.set_string(self.key_name, value)
 
-class _GConfTweak(Tweak):
-    def __init__(self, key_name, key_type, **options):
-        self.gconf = GConfSetting(key_name, key_type)
-        Tweak.__init__(self,
-            options.get("summary",self.gconf.schema_get_summary()),
-            options.get("description",self.gconf.schema_get_description()),
-            **options)
-
-class GConfComboTweak(_GConfTweak):
-    def __init__(self, key_name, key_type, key_options, **options):
-        _GConfTweak.__init__(self, key_name, key_type, **options)
-
-        #check key_options is iterable
-        #and if supplied, check it is a list of 2-tuples
-        assert len(key_options) >= 0
-        if len(key_options):
-            assert len(key_options[0]) == 2
-
-        combo = build_combo_box_text(
-            self.gconf.get_value(),
-            *key_options)
-        combo.connect('changed', self._on_combo_changed)
-        self.widget = build_label_beside_widget(self.name, combo)
-        self.widget_for_size_group = combo
-
-    def _on_combo_changed(self, combo):
-        _iter = combo.get_active_iter()
-        if _iter:
-            value = combo.get_model().get_value(_iter, 0)
-            self.gconf.set_value(value)
-
-class GConfFontButtonTweak(_GConfTweak):
-    def __init__(self, key_name, key_type, **options):
-        _GConfTweak.__init__(self, key_name, key_type, **options)
-
-        w = Gtk.FontButton()
-        w.props.font_name = self.gconf.get_value()
-        w.connect("notify::font-name", self._on_fontbutton_changed)
-        self.widget = build_label_beside_widget(self.name, w)
-        self.widget_for_size_group = w
-
-    def _on_fontbutton_changed(self, btn, param):
-        self.gconf.set_value(btn.props.font_name)
-
 class FileChooserButton(Gtk.FileChooserButton):
     def __init__(self, title, local_only, mimetypes):
         Gtk.FileChooserButton.__init__(self, title=title)
@@ -336,17 +401,18 @@ class FileChooserButton(Gtk.FileChooserButton):
         self.set_local_only(local_only)
         self.set_action(Gtk.FileChooserAction.OPEN)
 
-class GSettingsFileChooserButtonTweak(_GSettingsTweak):
-    def __init__(self, schema_name, key_name, local_only, mimetypes, **options):
+class GSettingsFileChooserButtonTweak(Gtk.Box, _GSettingsTweak, _DependableMixin):
+    def __init__(self, name, schema_name, key_name, local_only, mimetypes, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         _GSettingsTweak.__init__(self, schema_name, key_name, **options)
 
         self.settings.connect('changed::'+self.key_name, self._on_setting_changed)
 
-        self.filechooser = FileChooserButton(self.name,local_only,mimetypes)
+        self.filechooser = FileChooserButton(name,local_only,mimetypes)
         self.filechooser.set_uri(self.settings.get_string(self.key_name))
         self.filechooser.connect("file-set", self._on_file_set)
 
-        self.widget = build_label_beside_widget(self.name, self.filechooser)
+        build_label_beside_widget(name, self.filechooser, hbox=self)
         self.widget_for_size_group = self.filechooser
 
     def _values_are_different(self):
@@ -360,8 +426,29 @@ class GSettingsFileChooserButtonTweak(_GSettingsTweak):
         if uri and self._values_are_different():
             self.settings.set_string(self.key_name, uri)
 
-class DarkThemeSwitcher(Tweak):
+class GetterSetterSwitchTweak(Gtk.Box, Tweak):
+    def __init__(self, name, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
+        Tweak.__init__(self, name, options.get("description",""), **options)
+
+        sw = Gtk.Switch()
+        sw.set_active(self.get_active())
+        sw.connect("notify::active", self._on_toggled)
+
+        build_label_beside_widget(name, sw, hbox=self)
+
+    def _on_toggled(self, sw, pspec):
+        self.set_active(sw.get_active())
+
+    def get_active(self):
+        raise NotImplementedError()
+
+    def set_active(self, v):
+        raise NotImplementedError()
+
+class DarkThemeSwitcher(Gtk.Box, Tweak):
     def __init__(self, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
         Tweak.__init__(self, _("Enable dark theme for all applications"),
                        _("Enable the dark theme hint for all the applications in the session"),
                        **options)
@@ -370,9 +457,26 @@ class DarkThemeSwitcher(Tweak):
 
         w = Gtk.Switch()
         w.set_active(self._gtksettings.get_integer("gtk-application-prefer-dark-theme"))
-
+		
+        title = _("Global Dark Theme")
+        description = _("Applications need to be restarted for change to take effect")
         w.connect("notify::active", self._on_switch_changed)
-        self.widget = build_label_beside_widget(self.name, w)
+        
+        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        hbox.props.spacing = UI_BOX_SPACING
+        lbl = Gtk.Label(title)
+        lbl.props.ellipsize = Pango.EllipsizeMode.END
+        lbl.props.xalign = 0.0
+        hbox.pack_start(lbl, True, True, 0)
+        hbox.pack_start(w, False, False, 0)
+        
+        lbl_des = Gtk.Label()
+        lbl_des.props.xalign = 0.0
+        lbl_des.set_markup("<span size='x-small'>"+description+"</span>")
+        
+        self.pack_start(hbox, False, False, 0)
+        self.pack_start(lbl_des, False, False,0)
+        self.widget_for_size_group = None
 
     def _on_switch_changed(self, switch, param):
         active = switch.get_active()
@@ -382,3 +486,13 @@ class DarkThemeSwitcher(Tweak):
                                           active)
         except:
             self.notify_error(_("Error writing setting"))
+
+class Title(Gtk.Box, Tweak):
+    def __init__(self, name, desc, **options):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
+        Tweak.__init__(self, name, desc, **options)
+        widget = Gtk.Label()
+        widget.set_markup("<b>"+name+"</b>")
+        widget.props.xalign = 0.0
+        self.add(widget)
+
